/*
 * Serial port handling code
 */

#include "scif.h"

void scif_init(int baudrate)
{
  int clk,div;

  *(volatile unsigned short *)SCSCR2 = 0; //TX,RX disabled

  div=50000000/32/baudrate;

  for (clk=0;clk<4;clk++) {
  	if (div<=256) break;
  	div>>=2;
  }

  *(volatile unsigned short *)SCSCR2 = 0; //TX,RX disabled
  *(volatile unsigned short *)SCSMR2 = clk;  // 8 bits, 1 stop, no parity
  *(volatile unsigned char  *)SCBRR2 = div-1;
  *(volatile unsigned short *)SCFCR2 = 0xb6; //RX thr=8, TX thr=1, reset fifos
  *(volatile unsigned short *)SCFCR2 = 0xb0; // resets OFF
  *(volatile unsigned short *)SCSPTR2 = 0; //NO dumb io pins
  *(volatile unsigned short *)SCFSR2 = 0x60; // initial SR
  *(volatile unsigned short *)SCLSR2 = 0; // no overrun
  *(volatile unsigned short *)SCSCR2 = 0x30; //TX,RX enabled
}

void scif_send(unsigned char c)
{
  while((((*(volatile unsigned short *)SCFDR2)>>8) & 0x1f)==15);
  *(volatile unsigned char *)SCFTDR2=c;
  //*(volatile unsigned short *)SCFSR2 &=~0x20;
}

char scif_rec()
{
  char c;
  while(((*(volatile unsigned short *)SCFDR2) & 0x1f)==0);
  c=*(volatile unsigned char  *)SCFRDR2;
  //*(volatile unsigned short *)SCFSR2 &=~0x2;
  return c;
}

int scif_rec_check()
{
	return ((*(volatile unsigned short *)SCFDR2) & 0x1f);
}

int scif_write(unsigned char *buffer,int nb)
{
	int i;
	for (i=nb;i;i--) {
		while((((*(volatile unsigned short *)SCFDR2)>>8) & 0x1f)==15);
		*(volatile unsigned char *)SCFTDR2=*buffer++;
	}
	return nb;
}

int scif_read(unsigned char *buffer,int nb)
{
	int i;
	for (i=nb;i;i--) {
		while(((*(volatile unsigned short *)SCFDR2) & 0x1f)==0);
		*buffer++=*(volatile unsigned char  *)SCFRDR2;
	}
}

/* exploitable gets function */
void scif_gets(char *buf)
{
	do{
		*buf++=scif_rec();
	} while (buf[-1]!='\n');
	buf[-1]=0;
}

void scif_puts(char *buf)
{
	while (*buf!=0) scif_send(*buf++);
	//scif_send('\n');
}

char *rdnum(char *p, int *val)
{
	int i=0;

	while(*p==' ' || *p=='\t') p++;
	for(;;) {
		if ((*p)>'9' || (*p)<'0') break;
		i*=10;
		i+=(*p++)-'0';
	}
	*val=i;
	return p;
}


